/*
 * Copyright (c) 2007-2010 Sonatype, Inc. All rights reserved.
 *
 * "Sonatype" is a trademark of Sonatype, Inc.
 */

package org.sonatype.tycho.p2.publisher.plugin;

import java.io.File;
import java.io.FileWriter;
import java.net.URI;
import java.util.Arrays;
import java.util.Collection;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.assembly.model.Component;
import org.apache.maven.plugin.assembly.model.FileItem;
import org.apache.maven.plugin.assembly.model.FileSet;
import org.apache.maven.plugin.assembly.model.io.xpp3.ComponentXpp3Writer;
import org.apache.maven.project.MavenProject;
import org.codehaus.plexus.util.IOUtil;
import org.sonatype.tycho.p2.touchpoint.mixin.generic.impl.PermissionRegistry;
import org.sonatype.tycho.p2.touchpoint.mixin.generic.impl.PermissionRegistry.Entry;

/**
 * @goal generate-assembly-xml
 * @phase process-resources
 */
public class GenerateAssemblyXmlMojo
    extends AbstractMojo
{

    /**
     * @parameter expression="${project.build.directory}/tycho-p2/p2-install-folder"
     * @required
     */
    private File installDirectory;

    /**
     * @parameter expression="${project.build.directory}/tycho-p2/assembly-descriptor.xml"
     * @required
     */
    private File assemblyXml;

    /**
     * @parameter expression="${project.build.directory}"
     * @readonly
     */
    private File projectBuildDirectory;

    /**
     * @parameter expression="${project}"
     * @readonly
     */
    protected MavenProject project;

    /**
     * {@inheritDoc}
     */
    public void execute()
        throws MojoExecutionException
    {
        final URI projectDirUri = projectBuildDirectory.getParentFile().toURI();
        final URI assembleXmlUri = assemblyXml.toURI();
        final URI relativeUri = projectDirUri.relativize( assembleXmlUri );
        String generatedAssemblyXml = assemblyXml.toURI().toASCIIString();
        if ( !relativeUri.equals( assembleXmlUri ) )
        {
            generatedAssemblyXml = relativeUri.toASCIIString();
        }
        project.getProperties().setProperty( "tycho.p2.publisher.generatedAssemblyXml", generatedAssemblyXml );

        final Component component = new Component();
        final FileSet fileSet = new FileSet();
        fileSet.setDirectory( installDirectory.getAbsolutePath() );
        fileSet.setExcludes( Arrays.asList( "p2/org.eclipse.equinox.p2.core/cache/" ) );
        fileSet.setOutputDirectory( "/" );

        component.addFileSet( fileSet );

        final File dataArea = new File( installDirectory, "p2/org.sonatype.tycho.p2.touchpoint" );
        final File profileDirectory = new File( dataArea, "tycho-p2.profile" );
        final File permissionFile = new File( profileDirectory, "permissions" );
        if ( permissionFile.exists() )
        {
            final PermissionRegistry permissionRegistry = new PermissionRegistry( permissionFile );
            final Collection<Entry> permissions = permissionRegistry.getPermissions();
            for ( final Entry entry : permissions )
            {
                final FileItem fileItem = new FileItem();

                fileItem.setSource( new File( installDirectory, entry.getTarget().getPath() ).getAbsolutePath() );
                fileItem.setDestName( entry.getTarget().getPath() );
                fileItem.setFileMode( entry.getPermission() );

                component.addFile( fileItem );
            }
        }

        FileWriter writer = null;
        try
        {
            writer = new FileWriter( assemblyXml );
            new ComponentXpp3Writer().write( writer, component );
        }
        catch ( final Exception e )
        {
            IOUtil.close( writer );
        }
    }
}
